import axios from 'axios'
import { Modes, States } from '../constants/app'
import * as Urls from '../constants/resourcePaths'

const state = {
    currentMode: Modes.PURCHASE,
    currentState: States.NONE,
    footerButtonsVisibility: false,
    completed: false,
    loading: true
}

const getters = {
    mode() {
        return state.currentMode
    },
    isProcessing() {
        return state.currentState === States.PROCESSING
    },
    isDone() {
        return state.currentState === States.DONE
    },
    hasError() {
        return state.currentState === States.ERROR
    },
    state() {
        return state.currentState
    },
    isPurchase() {
        return state.currentMode === 'purchase'
    },
    isExpense() {
        return state.currentMode === 'expense'
    },
    isPurchasePicking() {
        return state.currentMode === 'product_picking'
    },
    isPurchasePayment() {
        return state.currentMode === 'payment'
    },
    isPurchaseTaking() {
        return state.currentMode === 'product_taking'
    },
    isCompleted(state) {
        return state.completed
    },
    isLoading() {
        return state.loading
    }
}

const mutations = {
    setMode(state, payload) {
        state.currentMode = payload
    },
    setState(state, payload) {
        state.currentState = payload
    },
    toggleFooterButtonsVisibility(state) {
        state.footerButtonsVisibility = !state.footerButtonsVisibility
    },
    setCompleted(state, completed) {
        state.completed = completed
    },
    setLoading(state, loading) {
        state.loading = loading
    }
}

const actions = {
    initProcess({ getters, commit, dispatch }, payload) {
        commit('setMode', payload || getters.mode)

        commit('setLoading', true)
        commit('setCompleted', false)
        
        return axios.get(Urls.INIT_PURCHASE_URL, {
            params: {
                mode: getters.mode
            }
        }).then(response => {
            commit('setLoading', false)
            commit('toggleFooterButtonsVisibility')
            
            dispatch('explodeData', response.data)
        }).catch(error => console.log(error))
    },
    explodeData({ dispatch }, payload) {
        for (let key in payload) {
            dispatch(`init${key[0].toUpperCase()}${key.slice(1)}`, payload[key])
        }
    },
    createObject({ _ }, payload) {
        const data = {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                type: payload.type,
                data: payload.data
            }
        }

        return axios.post(Urls.CREATE_OBJECT_URL, data).then(response => {
            console.log(response)
        }).catch(error => console.log(error))
    },
    checkCart({ getters, dispatch}) {
        for (let cartItem of getters.cartItems) {
            if (cartItem.price == 0) {
                return dispatch('notify', 'No puede haber items sin precio definido en el carrito')
            }
        }

        return !!getters.cartItems.length || dispatch('notify', 'Necesitar agregar producto al carrito para continuar')
    },
    checkSupplier({ getters, dispatch}) {
        return !!getters.selectedSupplier || dispatch('notify', 'Necesitas seleccionar un proveedor para continuar')
    },
    checkPurchaseOrder({ getters, dispatch }) {
        return !!getters.selectedPurchaseOrder || dispatch('notify', 'Necesitas seleccionar un presupuesto para continuar')
    },
    checkStockPicking({ getters, dispatch }) {
        return !!getters.selectedStockPicking || dispatch('notify', 'Necesitas seleccionar una entrega para continuar')
    },
    checkAmountReceived({ getters, dispatch }) {
        if (getters.paymentType == 'cash') {
            return getters.initialPayment >= getters.cartTotal || dispatch('notify', 'El monto recibido no puede ser menor al monto a pagar')
        } else {
            return getters.initialPayment < getters.cartTotal || dispatch('notify', 'El monto recibido no puede ser igual o mayor al monto a pagar')
        }
    },
    endProcess({ getters, commit, dispatch }) {
        const mode = getters.mode;

        if (mode == 'purchase') {
            if (getters.paymentType == 'cash' && getters.initialPayment < getters.amountToPay) {
                return dispatch('notify', 'El monto recibido no puede ser menor al monto a pagar')
            }

            if (getters.paymentType != 'cash' && getters.initialPayment >= getters.amountToPay) {
                return dispatch('notify', 'El monto recibido no puede ser igual o mayor al monto a pagar')
            }
        }

        commit('setLoading', true)

        let data = { mode }

        if (['purchase', 'expense', 'product_picking'].includes(mode)) {
            data = {
                ...data,
                items: getters.cartItems.map(item => {
                    return {
                        id: item.id,
                        name: item.name,
                        quantity: item.quantity,
                        price: item.price
                    }
                }),
                supplierId: getters.selectedSupplier.id,
                currencyId: getters.selectedCurrency.id,
                supplierInvoiceNumber: getters.supplierInvoiceNumber || null,
                warehouseId: getters.selectedWarehouse.id
            }
        }
        
        if (['purchase', 'expense', 'payment'].includes(mode)) {
            data = {
                ...data,
                items: getters.cartItems.map(item => {
                    return {
                        id: item.id,
                        name: item.name,
                        quantity: item.quantity,
                        price: item.price
                    }
                }),
                total: getters.cartTotal,
                purchaseOrderId: (getters.selectedPurchaseOrder && getters.selectedPurchaseOrder.id) || null,
                paymentTermId: getters.selectedPaymentTerm.id,
                journalId: getters.selectedJournal.id,
                payment: getters.initialPayment > getters.cartTotal ? getters.cartTotal : getters.initialPayment,
                paymentMethod: getters.paymentMethod,
                supplierId: (getters.selectedSupplier && getters.selectedSupplier.id) || null,
                currencyId: (getters.selectedCurrency && getters.selectedCurrency.id) || null,
                warehouseId: (getters.selectedWarehouse && getters.selectedWarehouse.id) || null
            }
        }

        if (mode == 'product_delivery') {
            data = {
                ...data,
                stockPickingId: getters.selectedStockPicking.id
            }
        }

        // const data = {
        //     jsonrpc: '2.0',
        //     method: 'call',
        //     params: {
        //         mode: getters.mode,
        //         items: getters.cartItems.map(item => {
        //             return {
        //                 id: item.id,
        //                 name: item.name,
        //                 quantity: item.quantity,
        //                 price: item.price
        //             }
        //         }),
        //         supplierId: getters.selectedSupplier.id,
        //         paymentTermId: getters.selectedPaymentTerm.id,
        //         journalId: getters.selectedJournal.id,
        //         payment: getters.initialPayment > getters.cartTotal ? getters.cartTotal : getters.initialPayment,
        //         supplierInvoiceNumber: getters.supplierInvoiceNumber || null
        //     }
        // }

        return axios.post(Urls.PROCESS_PURCHASE_URL, {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                ...data
            }
        }).then(_ => {
            dispatch('resetPurchase')

            commit('setLoading', false)
            commit('setCompleted', true)
        }).catch(error => {
            commit('setLoading', false)
        })
    },
    resetProcess({ rootState, dispatch }) {
        for (let key in rootState) {
            if (!(rootState[key] instanceof Object)) {
                continue
            }

            dispatch(`reset${key[0].toUpperCase()}${key.slice(1)}`)
        }

        dispatch('initProcess')
    },
    notify({ _ }, payload) {
        openerp.web.notification.do_warn('Atención', payload)
        return false
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}